require "import"
import "android.widget.*"
import "com.androlua.*"
import "android.view.View"
import "org.json.JSONObject"

-- Variable to store the current chat ID
currentChatId = nil

-- Function to make API request
function makeChatRequest(apiKey, modelId, prompt, maxTokens, chatId, callback)
  local url = "https://api.abhibhai.com/api/v6/llm/chat"
  local headers = {["Content-Type"] = "application/json"}
  
  local payload = {
    api_key = apiKey,
    model_id = modelId,
    prompt = prompt,
    max_new_tokens = maxTokens,
    do_sample = true,
    temperature = 0.3,
    top_k = 50,
    top_p = 0.95,
    no_repeat_ngram_size = 5,
    temp = false
  }
  
  if chatId and type(chatId) == "string" and #chatId > 0 then
    payload.chat_id = chatId
  end
  
  local json = require("cjson")
  local jsonPayload = json.encode(payload)
  
  Http.post(url, jsonPayload, headers, function(code, content)
    if code ~= 200 then
      callback(nil, "Error making API request: " .. code)
    else
      callback(content)
    end
  end)
end

-- Function to parse API response
function parseApiResponse(response)
  local json = require("cjson")
  local responseObj = json.decode(response)
  
  local aiMessage = responseObj.message or "No message found"
  local chatUrl = ""
  if responseObj.output and #responseObj.output > 0 then
    chatUrl = responseObj.output[1]
  end
  local chatId = responseObj.chat_id or ""
  
  return aiMessage, chatUrl, chatId
end

-- Create main layout
layout = {
  LinearLayout,
  orientation = "vertical",
  layout_width = "fill",
  layout_height = "fill",
  {
    EditText,
    id = "apiKeyInput",
    hint = "Enter your API key",
    layout_width = "fill",
    layout_height = "wrap_content",
    padding = "10dp",
  },
  {
    EditText,
    id = "modelIdInput",
    hint = "Enter model ID (e.g., zephyr-7b-beta)",
    layout_width = "fill",
    layout_height = "wrap_content",
    padding = "10dp",
  },
  {
    EditText,
    id = "promptInput",
    hint = "Enter your prompt",
    layout_width = "fill",
    layout_height = "wrap_content",
    padding = "10dp",
    lines = 3,
  },
  {
    EditText,
    id = "maxTokensInput",
    hint = "Max tokens (1-1024, default 128)",
    layout_width = "fill",
    layout_height = "wrap_content",
    padding = "10dp",
    inputType = "number",
  },
  {
    LinearLayout,
    orientation = "horizontal",
    layout_width = "fill",
    {
      Button,
      id = "sendButton",
      text = "Send Request",
      layout_width = "wrap_content",
      layout_height = "wrap_content",
    },
    {
      Button,
      id = "newChatButton",
      text = "New Chat",
      layout_width = "wrap_content",
      layout_height = "wrap_content",
    },
    {
      Button,
      id = "cancelButton",
      text = "Cancel",
      layout_width = "wrap_content",
      layout_height = "wrap_content",
    }
  },
  {
    ScrollView,
    layout_width = "fill",
    layout_height = "fill",
    {
      LinearLayout,
      orientation = "vertical",
      layout_width = "fill",
      layout_height = "wrap_content",
      {
        TextView,
        id = "chatIdText",
        text = "Chat ID: None",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "10dp",
      },
      {
        TextView,
        id = "aiResponseText",
        text = "",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "10dp",
      },
      {
        TextView,
        id = "chatUrlText",
        text = "",
        layout_width = "fill",
        layout_height = "wrap_content",
        padding = "10dp",
      }
    }
  },
  {
    TextView,
    text = "Powered by https://t.me/jcftbs",
    layout_width = "fill",
    layout_height = "wrap_content",
    padding = "10dp",
  }
}

-- Create dialog
dlg = LuaDialog(this)
dlg.setTitle("Chat open Tool")
dlg.setView(loadlayout(layout))

-- Function to update response
function updateResponse(aiMessage, chatUrl, chatId)
  aiResponseText.text = "AI Response:\n" .. aiMessage
  chatUrlText.text = "Chat URL:\n" .. chatUrl
  if chatId and type(chatId) == "string" and #chatId > 0 then
    currentChatId = chatId
    chatIdText.text = "Chat ID: " .. chatId
  end
end

-- Set up send button
sendButton.onClick = function()
  local apiKey = apiKeyInput.text
  local modelId = modelIdInput.text
  local prompt = promptInput.text
  local maxTokens = tonumber(maxTokensInput.text) or 128
  
  if apiKey and modelId and prompt and #apiKey > 0 and #modelId > 0 and #prompt > 0 then
    if maxTokens < 1 or maxTokens > 1024 then
      aiResponseText.text = "Max tokens should be between 1 and 1024"
      chatUrlText.text = ""
      return
    end
    
    aiResponseText.text = "Sending request..."
    chatUrlText.text = ""
    
    makeChatRequest(apiKey, modelId, prompt, maxTokens, currentChatId, function(response, error)
      if error then
        aiResponseText.text = "Error: " .. error
        chatUrlText.text = ""
      else
        local aiMessage, chatUrl, chatId = parseApiResponse(response)
        updateResponse(aiMessage, chatUrl, chatId)
      end
    end)
  else
    aiResponseText.text = "Please fill in all required fields"
    chatUrlText.text = ""
  end
end

-- Set up new chat button
newChatButton.onClick = function()
  currentChatId = nil
  chatIdText.text = "Chat ID: None"
  aiResponseText.text = ""
  chatUrlText.text = ""
  promptInput.text = ""
end

-- Set up cancel button
cancelButton.onClick = function()
  dlg.dismiss()
end

-- Show dialog
dlg.show()